/*-----------------------------------------------------------------------------
  Arrays class
  Written by Francois Fleuret <francois.fleuret@inria.fr>
  -----------------------------------------------------------------------------*/

#ifndef ARRAY_CC
#define ARRAY_CC

#include <stdio.h>

//-----------------------------------------------------------------------------

extern int arraySize;
void resetArraySize();
int getArraySize();

//-----------------------------------------------------------------------------

extern "C" void abort();

template <class T>
class Array
{
public:
  int size;
  T* body;

  Array(Array<T> &a);
  Array(int s);
  Array();

  ~Array();

  static void resetNb();
  static int getNb();

  void kill();
  void copy(Array &a);
  void resize(int t);
  void set(int p, T t);
  T get(int p);
  T* getref(int p);
  void absorb(Array<T> a);
};

template<class T>
inline Array<T>::Array(int s)
{
  size = s;
  body = new T[s];
#ifdef DEBUG
  arraySize += s*sizeof(T);
#endif
}

template<class T>
inline Array<T>::Array()
{
  size = -1; 
  body = NULL;
}

template<class T>
inline Array<T>::Array(Array<T> &a)
{
  int k;
#ifdef DEBUG
//  cerr<<"WARNING ARRAY DUPLICATION : "<<a.size<<" elements\n";
#endif
  //  abort();
  size = a.size;
  body = new T[size];
#ifdef DEBUG
  arraySize += size*sizeof(T);
#endif
  for(k=0; k<size; k++) body[k] = a.body[k];
}

template<class T>
inline Array<T>::~Array()
{
  if(size >= 0)
    {
      delete[] body;
#ifdef DEBUG
      arraySize -= size*sizeof(T);
      if(arraySize < 0)
	{
	  cerr<<"ARRAY DEALLOCATION ERROR\n";
	  abort();
	}
#endif
    }
}

template<class T>
inline void Array<T>::kill()
{
  if(size >= 0)
    {
      delete[] body;
#ifdef DEBUG
      arraySize -= size*sizeof(T);
      if(arraySize < 0)
	{
	  cerr<<"ARRAY DEALLOCATION ERROR\n";
	  abort();
	}
#endif
      size = -1;
    }
}

template<class T>
inline void Array<T>::copy(Array& a)
{
  int k;
  if(size >= 0)
    {
      delete[] body;
#ifdef DEBUG
      arraySize -= size*sizeof(T);
      if(arraySize < 0)
	{
	  cerr<<"ARRAY DEALLOCATION ERROR\n";
	  abort();
	}
#endif
      size = -1;
    }
  size = a.size;
  body = new T[size];
#ifdef DEBUG
  arraySize += size*sizeof(T);
#endif
  for(k=0; k<size; k++) body[k] = a.body[k];
}

template<class T>
inline void Array<T>::resize(int t)
{
  if(size >= 0)
    {
      delete[] body;
#ifdef DEBUG
      arraySize -= size*sizeof(T);
      if(arraySize < 0)
	{
	  cerr<<"ARRAY DEALLOCATION ERROR\n";
	  abort();
	}
#endif
    }
  size = t;
  body = new T[t];
#ifdef DEBUG
  arraySize += t*sizeof(T);
#endif
};

template<class T>
inline void Array<T>::set(int p, T t)
{
#ifdef DEBUG
  if((p<0) || (p>=size))
    {
      cerr<<"ARRAY ERROR in set() size = "<<size<<" indice = "<<p<<"\n";
      abort();
    }
#endif
  body[p] = t;
};

template<class T>
inline T Array<T>::get(int p)
{
#ifdef DEBUG
  if((p<0) || (p>=size))
    {
      cerr<<"ARRAY ERROR in get() size = "<<size<<" indice = "<<p<<"\n";
      abort();
    }
#endif
  return body[p];
};

template<class T>
inline T* Array<T>::getref(int p)
{
#ifdef DEBUG
  if((p<0) || (p>=size))
    {
      cerr<<"ARRAY ERROR in getref() size = "<<size<<" indice = "<<p<<"\n";
      abort();
    }
#endif
  return &body[p];
};

template<class T>
inline void Array<T>::absorb(Array<T> a)
{
  if(size >= 0)
    {
      delete[] body;
#ifdef DEBUG
      arraySize -= size*sizeof(T);
      if(arraySize < 0)
	{
	  cerr<<"ARRAY DEALLOCATION ERROR\n";
	  abort();
	}
#endif
    }
  size = a.size;
  body = a.body;
  a.size = -1;
  a.body = NULL;
};

//-----------------------------------------------------------------------------

#endif

//-----------------------------------------------------------------------------
